<!DOCTYPE html>
<html>
  <head>
    <meta http-equiv="Content-type" content="text/html; charset=utf-8"/>
    <title>Offset</title>
    <style>
      body {
        margin: 0px;
      }
      #wrapper {
        width: 300px;
        height: 300px;
        margin: 0px;
      }
      #clicker {
        position: relative;
        width: 100px;
        height: 100px;
        top: 100px;
        left: 100px;
        background-color: red;
        margin: 0px;
      }
    </style>
    <script src="/jquery.js" type="text/javascript" charset="utf-8"></script>
    <script src="/offset.js" type="text/javascript" charset="utf-8"></script>
  </head>
  <body>
    <div id="wrapper">
      <div id="clicker"></div>
    </div>
  </body>
  </html>
