<!DOCTYPE html>
<html>
    <head>
        <title>With Frames</title>
    </head>
    <body>
        <div id="divInMainWindow">
          This is the text for divInMainWindow
          <iframe src="/frame_parent" id="innerParentFrame"></iframe>
        </div>
    <iframe src="/frame_one" id="frameOne" name="my frame one"></iframe>
    <iframe src="/frame_two" id="frameTwo"></iframe>
    <iframe src="/frame_parent" id="parentFrame"></iframe>
    </body>
</html>
