<!DOCTYPE html>
<html>
  <head>
    <title>Namespace</title>
  </head>
  <body id="body">
    <div>
      <svg viewBox="0 0 100 100" preserveAspectRatio="xMidYMid slice"
           style="width:100%; height:100%; position:absolute; top:0; left:0; z-index:-1;">
        <linearGradient id="gradient">
          <stop class="begin" offset="0%"/>
          <stop class="end" offset="100%"/>
        </linearGradient>
        <rect x="0" y="0" width="100" height="100" style="fill:url(#gradient)" />
        <rect x="150" y="150" width="25" height="25" style="fill:url(#gradient)" />
        <circle cx="50" cy="50" r="30" style="fill:url(#gradient)" />
      </svg>
    <div>
  </body>
</html>
